@@ -1,20 +1,31 @@
 /* ocsp.c - OCSP (rfc2560)
- *      Copyright (C) 2003, 2004, 2005, 2006 g10 Code GmbH
+ * Copyright (C) 2003, 2004, 2005, 2006, 2012 g10 Code GmbH
  *
  * This file is part of KSBA.
  *
  * KSBA is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Fountion; either version 3 of the License, or
- * (at your option) any later version.
+ * it under the terms of either
  *
- * KSBA is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *   - the GNU Lesser General Public License as published by the Free
+ *     Software Foundation; either version 3 of the License, or (at
+ *     your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * or
+ *
+ *   - the GNU General Public License as published by the Free
+ *     Software Foundation; either version 2 of the License, or (at
+ *     your option) any later version.
+ *
+ * or both in parallel, as here.
+ *
+ * KSBA is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
+ * License for more details.
+ *
+ * You should have received a copies of the GNU General Public License
+ * and the GNU Lesser General Public License along with this program;
+ * if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -22,7 +33,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
-#include <errno.h>
 
 #include "util.h"
 
@@ -53,200 +63,6 @@ dump_hex (const unsigned char *p, size_t n)
 }
 #endif
 
-
-static  void
-parse_skip (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  if (ti->length)
-    {
-      assert (ti->length <= *len);
-      *len -= ti->length;
-      *buf += ti->length;
-    }
-}
-
-static gpg_error_t
-parse_sequence (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-    ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_SEQUENCE
-             && ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-  return err;
-}
-
-static gpg_error_t
-parse_enumerated (unsigned char const **buf, size_t *len, struct tag_info *ti,
-                  size_t maxlen)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-     ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_ENUMERATED
-             && !ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti->length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (maxlen && ti->length > maxlen)
-    err = gpg_error (GPG_ERR_TOO_LARGE);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-static gpg_error_t
-parse_integer (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-     ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_INTEGER
-             && !ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti->length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-static gpg_error_t
-parse_octet_string (unsigned char const **buf, size_t *len, struct tag_info *ti)
-{
-  gpg_error_t err;
-
-  err= _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-    ;
-  else if (!(ti->class == CLASS_UNIVERSAL && ti->tag == TYPE_OCTET_STRING
-             && !ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti->length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-
-/* Note that R_BOOL will only be set if a value has been given. Thus
-   the caller should set it to the default value prior to calling this
-   function.  Obviously no call to parse_skip is required after
-   calling this function. */
-static gpg_error_t
-parse_optional_boolean (unsigned char const **buf, size_t *len, int *r_bool)
-{
-  gpg_error_t err;
-  struct tag_info ti;
-
-  err = _ksba_ber_parse_tl (buf, len, &ti);
-  if (err)
-    ;
-  else if (!ti.length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti.length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-  else if (ti.class == CLASS_UNIVERSAL && ti.tag == TYPE_BOOLEAN
-           && !ti.is_constructed)
-    {
-      if (ti.length != 1)
-        err = gpg_error (GPG_ERR_BAD_BER);
-      *r_bool = !!**buf;
-      parse_skip (buf, len, &ti);
-    }
-  else
-    { /* Undo the read. */
-      *buf -= ti.nhdr;
-      *len += ti.nhdr;
-    }
-
-  return err;
-}
-
-
-
-static gpg_error_t
-parse_object_id_into_str (unsigned char const **buf, size_t *len, char **oid)
-{
-  struct tag_info ti;
-  gpg_error_t err;
-
-  *oid = NULL;
-  err = _ksba_ber_parse_tl (buf, len, &ti);
-  if (err)
-    ;
-  else if (!(ti.class == CLASS_UNIVERSAL && ti.tag == TYPE_OBJECT_ID
-                && !ti.is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!ti.length)
-    err = gpg_error (GPG_ERR_TOO_SHORT);
-  else if (ti.length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-  else if (!(*oid = ksba_oid_to_str (*buf, ti.length)))
-    err = gpg_error_from_errno (errno);
-  else
-    {
-      *buf += ti.length;
-      *len -= ti.length;
-    }
-  return err;
-}
-
-
-static gpg_error_t
-parse_asntime_into_isotime (unsigned char const **buf, size_t *len,
-                            ksba_isotime_t isotime)
-{
-  struct tag_info ti;
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, &ti);
-  if (err)
-    ;
-  else if ( !(ti.class == CLASS_UNIVERSAL
-              && (ti.tag == TYPE_UTC_TIME || ti.tag == TYPE_GENERALIZED_TIME)
-              && !ti.is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (!(err = _ksba_asntime_to_iso (*buf, ti.length,
-                                         ti.tag == TYPE_UTC_TIME, isotime)))
-    parse_skip (buf, len, &ti);
-
-  return err;
-}
-
-
-static gpg_error_t
-parse_context_tag (unsigned char const **buf, size_t *len, struct tag_info *ti,
-                   int tag)
-{
-  gpg_error_t err;
-
-  err = _ksba_ber_parse_tl (buf, len, ti);
-  if (err)
-    ;
-  else if (!(ti->class == CLASS_CONTEXT && ti->tag == tag
-	     && ti->is_constructed) )
-    err = gpg_error (GPG_ERR_INV_OBJ);
-  else if (ti->length > *len)
-    err = gpg_error (GPG_ERR_BAD_BER);
-
-  return err;
-}
-
-
 
 /* Create a new OCSP object and retrun it in R_OCSP.  Return 0 on
    success or an error code.
@@ -256,7 +72,7 @@ ksba_ocsp_new (ksba_ocsp_t *r_ocsp)
 {
   *r_ocsp = xtrycalloc (1, sizeof **r_ocsp);
   if (!*r_ocsp)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   return 0;
 }
 
@@ -327,7 +143,7 @@ ksba_ocsp_set_digest_algo (ksba_ocsp_t ocsp, const char *oid)
     xfree (ocsp->digest_oid);
   ocsp->digest_oid = xtrystrdup (oid);
   if (!ocsp->digest_oid)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   return 0;
 }
 
@@ -356,7 +172,7 @@ ksba_ocsp_add_target (ksba_ocsp_t ocsp,
 
   ri = xtrycalloc (1, sizeof *ri);
   if (!ri)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   ksba_cert_ref (cert);
   ri->cert = cert;
   ksba_cert_ref (issuer_cert);
@@ -389,10 +205,6 @@ ksba_ocsp_set_nonce (ksba_ocsp_t ocsp, unsigned char *nonce, size_t noncelen)
   if (noncelen)
     {
       memcpy (ocsp->nonce, nonce, noncelen);
-      /* Reset the high bit.  We do this to make sure that we have a
-         positive integer and thus we don't need to prepend a leading
-         zero which would be needed then. */
-      ocsp->nonce[0] &= 0x7f;
     }
   ocsp->noncelen = noncelen;
   return noncelen;
@@ -480,7 +292,7 @@ write_request_extensions (ksba_ocsp_t ocsp, ksba_writer_t wout)
     err = _ksba_ber_write_tl (w1, TYPE_OCTET_STRING, CLASS_UNIVERSAL, 0,
                               2+ocsp->noncelen);
   if (!err)
-    err = _ksba_ber_write_tl (w1, TYPE_INTEGER, CLASS_UNIVERSAL, 0,
+    err = _ksba_ber_write_tl (w1, TYPE_OCTET_STRING, CLASS_UNIVERSAL, 0,
                               ocsp->noncelen);
   if (!err)
     err = ksba_writer_write (w1, ocsp->nonce, ocsp->noncelen);
@@ -607,8 +419,6 @@ ksba_ocsp_prepare_request (ksba_ocsp_t ocsp)
 
       /* Write the serialNumber of the certificate to be checked. */
       err = _ksba_cert_get_serial_ptr (ri->cert, &der, &derlen);
-      if (!err)
-        err = _ksba_ber_write_tl (w1, TYPE_INTEGER, CLASS_UNIVERSAL, 0, derlen);
       if (!err)
         err = ksba_writer_write (w1, der, derlen);
       if (err)
@@ -616,9 +426,10 @@ ksba_ocsp_prepare_request (ksba_ocsp_t ocsp)
       xfree (ri->serialno);
       ri->serialno = xtrymalloc (derlen);
       if (!ri->serialno)
-        err = gpg_error_from_errno (errno);
-      if (err)
-        goto leave;
+        {
+          err = gpg_error_from_syserror ();
+          goto leave;
+        }
       memcpy (ri->serialno, der, derlen);
       ri->serialnolen = derlen;
 
@@ -894,7 +705,7 @@ parse_response_extensions (ksba_ocsp_t ocsp,
         goto leave;
       if (!strcmp (oid, oidstr_ocsp_nonce))
         {
-          err = parse_integer (&data, &datalen, &ti);
+          err = parse_octet_string (&data, &datalen, &ti);
           if (err)
             goto leave;
           if (ocsp->noncelen != ti.length
@@ -906,7 +717,7 @@ parse_response_extensions (ksba_ocsp_t ocsp,
       ex = xtrymalloc (sizeof *ex + strlen (oid) + ti.length);
       if (!ex)
         {
-          err = gpg_error_from_errno (errno);
+          err = gpg_error_from_syserror ();
           goto leave;
         }
       ex->crit = is_crit;
@@ -973,7 +784,7 @@ parse_single_extensions (struct ocsp_reqitem_s *ri,
       ex = xtrymalloc (sizeof *ex + strlen (oid) + ti.length);
       if (!ex)
         {
-          err = gpg_error_from_errno (errno);
+          err = gpg_error_from_syserror ();
           goto leave;
         }
       ex->crit = is_crit;
@@ -1415,7 +1226,7 @@ parse_response_data (ksba_ocsp_t ocsp,
         return gpg_error (GPG_ERR_INV_OBJ); /* Zero length key id.  */
       ocsp->responder_id.keyid = xtrymalloc (ti.length);
       if (!ocsp->responder_id.keyid)
-        return gpg_error_from_errno (errno);
+        return gpg_error_from_syserror ();
       memcpy (ocsp->responder_id.keyid, *data, ti.length);
       ocsp->responder_id.keyidlen = ti.length;
       parse_skip (data, datalen, &ti);
@@ -1578,12 +1389,12 @@ parse_response (ksba_ocsp_t ocsp, const unsigned char *msg, size_t msglen)
         parse_skip (&msg, &msglen, &ti);
         cl = xtrycalloc (1, sizeof *cl);
         if (!cl)
-          err = gpg_error_from_errno (errno);
-        if (err)
           {
+            err = gpg_error_from_syserror ();
             ksba_cert_release (cert);
-            return gpg_error (GPG_ERR_ENOMEM);
+            return err;
           }
+
         cl->cert = cert;
 
         *cl_tail = cl;
@@ -1737,7 +1548,7 @@ ksba_ocsp_get_responder_id (ksba_ocsp_t ocsp,
     {
       *r_name = xtrystrdup (ocsp->responder_id.name);
       if (!*r_name)
-        return gpg_error_from_errno (errno);
+        return gpg_error_from_syserror ();
     }
   else if (ocsp->responder_id.keyid && r_keyid)
     {
@@ -1748,7 +1559,7 @@ ksba_ocsp_get_responder_id (ksba_ocsp_t ocsp,
       numbuflen = strlen (numbuf);
       *r_keyid = xtrymalloc (numbuflen + ocsp->responder_id.keyidlen + 2);
       if (!*r_keyid)
-        return gpg_error_from_errno (errno);
+        return gpg_error_from_syserror ();
       strcpy (*r_keyid, numbuf);
       memcpy (*r_keyid+numbuflen,
               ocsp->responder_id.keyid, ocsp->responder_id.keyidlen);
@@ -1756,7 +1567,7 @@ ksba_ocsp_get_responder_id (ksba_ocsp_t ocsp,
       (*r_keyid)[numbuflen + ocsp->responder_id.keyidlen + 1] = 0;
     }
   else
-    gpg_error (GPG_ERR_NO_DATA);
+    return gpg_error (GPG_ERR_NO_DATA);
 
   return 0;
 }
